#include "uart.h"
#include "main.h"

unsigned char DataRX;

char CommandPosition = 0;
char ArgumentPosition = 0;

char CommandComplete = 0;

char RX_CMD[4] = {0,0,0,0};
char RX_ARG[64];

#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
  _asm GOTO high_isr _endasm
}
#pragma code /* return to the default code section */
#pragma interrupt high_isr

void high_isr (void)
{
  //Load our DataRX byte holder with incoming byte
  DataRX = RCREG;
  
  //If We are busy with a command already skip input, must be cleared after a command 
  if(CommandComplete == 1)
  {
    PIR1 = 0; //Clear the interrupt flag
    return;
  }
   

  //Load 4 bytes into our command array
  if(CommandPosition < 4)
  {
    RX_CMD[CommandPosition++] = DataRX;
    PIR1 = 0; //Clear the interrupt flag
    return;
  }

  //Get our ARGUMENT array ready by clearing it.
  if(CommandPosition++ <= 4)
  {
	memset(RX_ARG,0,128);
    PIR1 = 0; //Clear the interrupt flag
    return;
  }

  //if data in is a LineFeed then its the end of argument
  if(DataRX == LF)
  {
    ArgumentPosition = 0;
    CommandPosition = 0;
    CommandComplete = 1;
    PIR1 = 0; //Clear the interrupt flag
    return;
  }
  //Load bytes into the argument array
  RX_ARG[ArgumentPosition++] = DataRX;
  PIR1 = 0; //Clear the interrupt flag
}

void Init_UART(void)
{
    TRISC  = 0b10000000;	//UART RX is INPUT mode

    //BAUDCONbits.RXDTP = 1;	//RX data is inverted
	BAUDCTLbits.BRG16 = 0;	//SPBRG only (Compatible mode), SPBRGH value ignored
    TXSTAbits.SYNC = 0; 	//Asynchronous mode


	if(SPEED_0 == 1)
	{
		//19200bps
    	TXSTAbits.BRGH = 1;		//High speed
		SPBRG = 129;     
	}
	else
	{
		//9600bps
    	TXSTAbits.BRGH = 0;		//Low speed
		SPBRG = 64;     
	}

    RCONbits.IPEN = 1;		//Enable priority levels on interrupts
    TXSTAbits.TXEN = 1;  	//ENABLE TX
    RCSTAbits.CREN = 1;  	//ENABLE RX
    RCSTAbits.SPEN = 1;  	//ENABLE SERIAL PORT and PIN Config

    PIE1bits.RCIE = 1;      //ENABLE RX Interrupt
    PIR1   = 0b00000000;	//Clear all flags
    PIE1   = 0b00100000;	//EUSART Receive Interrupt Enable bit
    IPR1   = 0b00100000;	//EUSART Receive Interrupt Priority bit (HIGH)
    INTCON = 0b11000000;	//Enables all high-priority interrupts,Enables all low-priority peripheral interrupts
}


void UART_TX(char dataOut)
{
  TXREG = dataOut;
  while(TXSTAbits.TRMT==0);    //Wait until Transmit is Complete
  Delay10TCYx(1);
}

void UART_STR(char* dataOut)
{
  while(*dataOut)
  {
    UART_TX(*dataOut++);
  }
}